/* Program  text.cpp

   c Petr Mikulik, Masaryk University Brno, mikulik@physics.muni.cz

   Shows all printable strings from a binary file.
   The program is aware of different encodings (pure 7bit ascii, CP852,
   ISO-Latin...). The encoding table comes from the program cs2cs by
   Ferdinand Prantl, Jr., thanks.

   History:
	16.1.1997 --- bugfix release (forgotten Inp.close())
	23.8.1996 --- program written
   Version:

*/
#define DATE "16.1.1997"
/*
  Copying policy: this file is in public domain.
  Please distribute it together with the source code.
*/

/*
Compilation:
OS/2 and emx/gcc:
  gcc -s -O2 -m486 -o text.exe text.cpp -lstdcpp
  (OS/2 and DOS targets)
Unix and g++:
  g++ -s -O2 -o text text.cpp
*/



#include <stdlib.h>
#include <string.h>
#include <fstream.h>
#include <iomanip.h>

#define DefaultLen 4

// command line options
char verbose=0;  // be verbose. Unused option
int len=DefaultLen;       // minimal string length
char debug=0;    // debugging mode
char nonASCII=0; // index to Encoding, 0 if only pure 7 bit ASCII allowed
char *Encoding;  // pointer to the Encoding table
char offseton=0; // print offset

// declarations:
ifstream Inp; // ofstream out;
unsigned char c;
long wordlen, Offset, OffsetX;
char *w; // save part of word here


char *EncodingTable[] = {
  "CP852",      "Ґ♚Ԃء眣짖꒓",
  "CP850",      "CDENRSTUZLRLOOUAScdenrstuzlrloouas&",
  "CP1250",     "؊ݎħ&",
  "IBM852",     "Ґ♚Ԃء眣즖꒓&",
  "ISO-8859-1", "ACDENRSTUZLRLOOUAScdenrstuzlrloouas&",
  "ISO-8859-2", "ةݮSs&",
  "KEYBCS2",    "&",
  "KOI8-CS",    "Ss&",
  "MAC",        "牑LRLOOUAS˗lrloouas&",
  "CORK",       "Ʌ͌ӐڗݚSᣤ󰲴s&",
  "" };


#if defined(__EMX__)
  #define __os2
  #define __OS2__
#endif

char ischar ( char c ) {
//return (c>31 && c<127);
return ( (c>31 && c<127) || (nonASCII && strchr(Encoding,c)) );
}

void printword ( ) {
if (offseton) cout<<setw(4)<<hex<<OffsetX<<": ";
for (int i=1; i<=wordlen; cout<<w[i++]);
}

#if defined(__os2) || defined(__OS2__) || defined(__MSDOS__)
  #define JA "Petr Mikulk"
#else
  #define JA "Petr Mikulik"
#endif

void PrintTitle () {
cerr<<"Program  text  by "<<JA<<" (version "<<DATE<<")\n";
cerr<<"for extracting text strings from binary files.\n";
cerr<<"This program is freeware, C++ source code at http://www.sci.muni.cz/~mikulik/\n\n";
}

void Usage () {
PrintTitle();
cerr<<"Usage:  text  [options]  input_file(s)\n";
cerr<<"Options:\n";
cerr<<"  -h, -?    ... help\n";
cerr<<"  -n        ... where n is a number denoting the minimal string length\n";
cerr<<"                (default "<<DefaultLen<<")\n";
cerr<<"  -p        ... show the string offset\n";
cerr<<"  -d        ... debug\n";
// cerr<<"-v  ... be verbose (unused)\n";
cerr<<"  -encoding ... where the encoding added to the default 7bit ascii 31<x<127 is:";
for (int i=0; *EncodingTable[i]; i+=2) {
  if (i%12==0) cerr<<"\n              ";
  cerr<<"  -"<<EncodingTable[i]; }
cerr<<endl;
exit(1);
}


// ********************** main *****************************

int main( int argc, char *argv[] ) {
if (argc==1) Usage();

// wildcard expansion: automatically under Unix, here for gcc/emx
#ifdef __EMX__
_wildcard(&argc,&argv);
#endif

/*
signed char copt;
while ((copt = pmgetopt (argc, argv, "v0123456789?h")) != EOF) {
  if (copt>='0' && copt<='9') {
    char *cptr=optplace-1;
    len=strtol(cptr,&optplace,10);
    //    if (!*optplace) optind++;
    }
else
  switch (copt) {
    case '?': case 'h': Usage(); break;
    case 'v': verbose=1; break;
	      //optopt+=strlen(argv[optind])-2;
    }
  }
*/

// if (!verbose) PrintTitle();

int i, optind=1;
char *cptr;
if (argc==1) Usage();
for (optind=1; optind<argc; optind++) {
  if (*argv[optind]!='-') break;
  // is it encoding?
  cptr=argv[optind]+1;
  if (!strcmp("d",cptr)) debug=1; else
  if (!strcmp("p",cptr)) offseton=1; else
  if (!strcmp("v",cptr)) verbose=1; else
  if (!strcmp(cptr,"?") || !strcmp(cptr,"h")) Usage(); else {
  // is it encoding?
  for (i=0; *EncodingTable[i]; i+=2)
    if (!strcmp(EncodingTable[i],cptr)) {
      Encoding=EncodingTable[nonASCII=i+1];
      break;
      }
  if (!*EncodingTable[i]) {
  // is it number, i.e. string length?
  len=strtol(cptr,&cptr,10);
  if (*cptr) {
    cerr<<"unknown option: "<<argv[optind]<<endl;
    exit(1); }
  } } }

if (optind>=argc)
  { cerr << "Input file not given.\n"; exit(1); }
// inputfile=
// if (optind<argc) outputfile=optind++;

w=new char [len+1]; // save part of word here

for ( ; optind<argc; optind++) { // for all input files

// cerr<<"***** I="<<optind<<"  FILE="<<argv[optind]<<endl;

// open file in binary mode
#ifdef __GNUC__
  Inp.open(argv[optind],ios::in /*|ios::bin*/);
#else
  Inp.open(argv[optind],ios::binary);
#endif
if (!Inp) { 
  cerr << "\nWarning: cannot open file "<<argv[optind]<<" (maybe directory?)\n";
  continue;
  // return 1; 
  };

Offset=0;
wordlen=0;

if (debug) {
cerr<<"\nMinimal text length: "<<len<<endl;
cerr<<"Encoding table: pure ASCII (31<n<127)";
if (nonASCII)
  cerr<<" + "<<EncodingTable[nonASCII-1]<<" ("<<Encoding<<')';
cerr<<"\n\n";
}

while (!Inp.eof() ) {
  // Inp >> c;  // maybe unefficient, but who takes care...
  Inp.read(&c,1);
  if (!Inp) {
    if (wordlen>=len) cout<<endl;
    break;
    }
  if (c && ischar(c)) {
      wordlen++;
      if (offseton && wordlen==1) OffsetX=Offset;
      if (wordlen>len) cout<<c; else
	if (wordlen<len) w[wordlen]=c; else
	  { w[wordlen]=c; printword(); }
      }
    else {
      if (wordlen>=len) cout<<endl;
      wordlen=0;
      }
  Offset++;
  }
Inp.close();
} // end for all input files

return 0;
}
// eof text.cpp

